function [val] = Rayleigh_tensor(A,X,P,D)
% MATRIX: For a matrix A, Rayleigh_tensor evaluates the Rayleigh quotient 
% associated to the (p,q)-norm matrix problem, i.e. it evaluates the expression
%
%                      (y'*A*z)/(||y||_p*||z||_q)
%
% where ||y||_p = norm(y,p) and ||z||_q = norm(z,q) (see below). Note that
% a matrix is a tensor of order 2 and the following explanations are
% directly made for the more general case of tensors.
%
% TENSOR: For a tensor A of order m, Rayleigh_tensor evaluates the Rayleigh 
% quotient associated to the tensor projective (p_1,...,p_m)-norm problem, 
% i.e. it evaluates the expression
%                   
%      A(x_1,...,x_m)/(||x_1||_{p_1}*||x_2||_{p_2}*...*||x_m||_{p_m}) 
%
% where:
% - A(x_1,...,x_m) is the multilinear form associated to the tensor A 
%   evaluated at x_1 in R^{d_1}, x_2 in R^{d_2}, ... , x_m in R^{d_m}, i.e.
% A(x)=sum_{0<j_1<d_1+1,...,0<j_m<d_m+1}A(j_1,...,j_m)*x_1(j_1)*...*x_m(j_m) 
%
% - ||x_i||_{p_i} is the p_i-norm on R^{d_i}, i.e. ||x_i||_{p_i}=norm(x_i,p_i)
%
% INPUT:
% - A = Tensor in matlab format (see example of use). For tensor input in
%       vector format see input D.
%
% - X = vector of size sum(size(A)) containing the vectors at which the
%       Rayleigh quotient should be evaluated, i.e. X = [x_1 x_2 ... x_m].
%
% - (OPTIONAL): P = row vector containing the constraints parameter
%                   p_1,...,p_m. Default value is p_1=...=p_m=m.
%
% - (OPTIONAL): D = Dimensions of the tensor. If you wish to give A as a
%                   vector containing the entries of the tensor in the
%                   reversed lexicographic order, precise its dimensions in
%                   the vector of integer D (having length equal to the
%                   order of the tensor).
%
% OUTPUT:
% - val = value of the Rayleigh quotient associated to the tensor projective 
%         (p_1,...,p_m)-norm of A evaluated at X.
%
% EXAMPLES OF USE:
%  - Ex. 1:
%  [val]=Rayleigh_tensor(rand(2,3),rand(1,5),[8.6,4.9])            
%
%  - Ex. 2:
%  [val]=Rayleigh_tensor(randn(10,11,25),randn(1,46),[2.5,8,3])
%
%  - Ex. 4:
%  [val]=Rayleigh_tensor(rand(1,2*3*5*8*2),ones(1,2+3+5+8+2),...
%                        [5.6,8.1,5.2,5.3,12.2],[2,3,5,8,2])

%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%

% Filling missing arguments
%
if nargin < 4
    D=size(A);
end
d = length(D);
if nargin < 3
    P=d*ones(1,d);
else
    if length(P) ~= d
        error('The vector P containing p_1,p_2,...,p_m should be the same size as the order of the tensor');
    end
end
if length(X)~= sum(D)
     error('The vector X does not have the correct dimension, i.e. length(starting_points) =/= sum(size(A))');
end

% Evaluates the Rayleigh quotient
%
ji_min = 1;
ji_max = D(1);

num = dot(single_supergrad(A,X,D,0),X(ji_min:ji_max));

quot = 1;
for i=1:d
    quot = quot*norm(X(ji_min:ji_max),P(i));
    if i < length(D)
        ji_min = ji_min + D(i);
        ji_max = ji_max + D(i+1);
    end
end

val = num/quot;

